@@ -233,7 +233,7 @@ stat_dens2d_filter <-
     if (any(is.na(keep.number) | keep.number < 0)) {
       stop("Out of range or missing value for 'keep.number': ", keep.number)
     }
-    max.expected.length <- c(none = 4L, x = 2L, y = 2L, xy = 1L)[pool.along]
+    max.expected.length <- unname(c(none = 4L, x = 2L, y = 2L, xy = 1L)[pool.along])
     if (length(keep.fraction) > max.expected.length) {
       if (max.expected.length == 4L) {
         stop("Length of 'keep.fraction' should not exceed 4")
@@ -297,13 +297,13 @@ stat_dens2d_filter_g <-
 
     pool.along <- rlang::arg_match(pool.along)
 
-    if (is.na(keep.fraction) || keep.fraction < 0 || keep.fraction > 1) {
+    if (any(is.na(keep.fraction) | keep.fraction < 0 | keep.fraction > 1)) {
       stop("Out of range or missing value for 'keep.fraction': ", keep.fraction)
     }
-    if (is.na(keep.number) || keep.number < 0) {
+    if (any(is.na(keep.number) | keep.number < 0)) {
       stop("Out of range or missing value for 'keep.number': ", keep.number)
     }
-    max.expected.length <- c(none = 4L, x = 2L, y = 2L, xy = 1L)[pool.along]
+    max.expected.length <- unname(c(none = 4L, x = 2L, y = 2L, xy = 1L)[pool.along])
     if (length(keep.fraction) > max.expected.length) {
       if (max.expected.length == 4L) {
         stop("Length of 'keep.fraction' should not exceed 4")
